"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ipfs_1 = __importDefault(require("ipfs"));
const Utils_1 = require("./Utils");
const dc_logging_1 = require("@daocasino/dc-logging");
const dc_configs_1 = require("@daocasino/dc-configs");
const defaultSwarm = dc_configs_1.config.default.transportServersSwarm[dc_configs_1.TransportType.IPFS];
const logger = new dc_logging_1.Logger("createIpfsNode");
function createIpfsNode(Swarm = defaultSwarm, attempt = 0) {
    const promise = new Promise((resolve, reject) => {
        const errors = [];
        const ipfs = new ipfs_1.default({
            repo: Utils_1.getRepoPath(),
            EXPERIMENTAL: {
                pubsub: true
            },
            config: {
                Addresses: {
                    Swarm
                }
            }
        })
            .on("start", () => {
            const id = ipfs.id().then(info => {
                ipfs.id = info.id;
                resolve(ipfs);
            });
        })
            .on("error", error => {
            errors.push(error);
            reject(error);
        });
    });
    return promise.catch(error => {
        if (attempt >= Swarm.length - 1) {
            logger.info(`All signal server in swarm raised errors`);
            throw error;
        }
        logger.warn(`Error creating ipfs node, swarm top ${Swarm[0]}`);
        logger.warn(error);
        Swarm.push(Swarm.shift());
        logger.info(`Trying to create ipfs node with swarm top ${Swarm[0]}`);
        return createIpfsNode(Swarm, attempt + 1);
    });
}
exports.createIpfsNode = createIpfsNode;
function destroyIpfsNode(ipfs) {
    return new Promise((resolve, reject) => {
        ipfs.stop(error => {
            if (error) {
                reject(error);
            }
            resolve();
        });
    });
}
exports.destroyIpfsNode = destroyIpfsNode;
//# sourceMappingURL=Ipfs.js.map