"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mocha_1 = require("mocha");
const chai_1 = require("chai");
const dc_logging_1 = require("@daocasino/dc-logging");
const dc_configs_1 = require("@daocasino/dc-configs");
const TransportProviderFactory_1 = require("../TransportProviderFactory");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const log = new dc_logging_1.Logger("Transport provider");
class TestService {
    sum(a, b) {
        return a + b;
    }
    checkFiles(info) {
        const files = readFiles(__dirname);
        // console.log({ remote: info.files, original: files.files })
        chai_1.expect(info).to.deep.equal(files);
    }
    start(address, provider) {
        this._provider = provider;
        this._address = address;
        provider.exposeSevice(address, this, false);
        log.debug(`Test service started at "${address}".`);
    }
    stop() {
        return this._provider.stopService(this._address);
    }
}
const readFiles = (openDir) => {
    const files = fs_1.default.readdirSync(openDir).filter(fileName => {
        const filePath = path_1.default.join(openDir, fileName);
        return fs_1.default.statSync(filePath).isFile();
    }).map(fileName => {
        const filePath = path_1.default.join(openDir, fileName);
        const { size } = fs_1.default.statSync(filePath);
        return { fileName, fileData: fs_1.default.readFileSync(filePath).toString('base64'), fileSize: size };
    });
    return { files };
};
const test = (factory) => mocha_1.describe(`Transport provider: ${factory.toString()}`, () => {
    mocha_1.it('Create TestService', async () => {
        const ROOM_ADDRESS = 'test';
        const serviceProvider = await factory.create();
        const testService = new TestService();
        testService.start(ROOM_ADDRESS, serviceProvider);
        chai_1.expect(testService.sum(1, 1)).to.be.equal(2);
        const files = readFiles(__dirname);
        testService.checkFiles(files);
        const remoteProvider = await factory.create();
        const remoteTestService = await remoteProvider.getRemoteInterface(ROOM_ADDRESS, "Remote TestService interface");
        try {
            const result = await remoteTestService.sum(1, 1);
            chai_1.expect(result).to.be.equal(2);
            remoteTestService.checkFiles(files);
        }
        catch (e) {
            log.error(e);
        }
        testService.stop();
        remoteProvider.destroy();
        serviceProvider.destroy();
    });
});
if (Object.values(dc_configs_1.TransportType).includes(process.env.DC_TRANSPORT)) {
    test(new TransportProviderFactory_1.TransportProviderFactory(dc_configs_1.TransportType[process.env.DC_TRANSPORT]));
}
else {
    Object.values(dc_configs_1.TransportType).forEach(key => {
        if (typeof key === 'number') {
            test(new TransportProviderFactory_1.TransportProviderFactory(key));
        }
    });
}
//# sourceMappingURL=TransportProvider.test.js.map