"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mock_ipfs_1 = __importDefault(require("@daocasino/mock-ipfs"));
const dc_logging_1 = require("@daocasino/dc-logging");
const dc_configs_1 = require("@daocasino/dc-configs");
const defaultSwarm = dc_configs_1.config.default.transportServersSwarm[dc_configs_1.TransportType.LIBP2P];
const logger = new dc_logging_1.Logger("createMockIpfsNode");
function createMockIpfsNode(Swarm = defaultSwarm, attempt = 0) {
    const promise = new Promise((resolve, reject) => {
        const errors = [];
        const ipfs = new mock_ipfs_1.default(Swarm, {
            config: {
                EXPERIMENTAL: {
                    pubsub: true
                },
            }
        })
            .on("start", () => {
            const id = ipfs.id().then(info => {
                ipfs.id = info.id;
                resolve(ipfs);
            });
        })
            .on("error", error => {
            errors.push(error);
            reject(error);
        });
    });
    return promise.catch(error => {
        if (attempt >= Swarm.length - 1) {
            logger.info(`All signal server in swarm raised errors`);
            throw error;
        }
        logger.warn(`Error creating libp2p node, swarm top ${Swarm[0]}`);
        logger.warn(error);
        Swarm.push(Swarm.shift());
        logger.info(`Trying to create libp2p node with swarm top ${Swarm[0]}`);
        return createMockIpfsNode(Swarm, attempt + 1);
    });
}
exports.createMockIpfsNode = createMockIpfsNode;
function destroyMockIpfsNode(mockIpfs) {
    return new Promise((resolve, reject) => {
        mockIpfs.stop(error => {
            if (error) {
                reject(error);
            }
            resolve();
        });
    });
}
exports.destroyMockIpfsNode = destroyMockIpfsNode;
//# sourceMappingURL=MockIpfs.js.map