"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
let _id = 0;
exports.getId = () => {
    return Date.now() * 1000000 + _id++;
};
class RemoteProxy {
    constructor() {
        this._requestCallbacks = new Map();
        this.onMessage = this.onMessage.bind(this);
        this._subscriptions = new Map();
    }
    getProxy(sendRequest, timeout = 15000) {
        const _self = this;
        const proxy = new Proxy({}, {
            get: (target, prop) => {
                if (prop === "then")
                    return null;
                if (prop === "constructor")
                    return null;
                if (prop === "inspect")
                    return null;
                if (typeof prop === 'symbol')
                    return null;
                if (prop === "on")
                    return _self._handleSubscribe.bind(_self);
                if (prop && prop.startsWith && !prop.startsWith("_")) {
                    return async (...params) => {
                        const id = exports.getId();
                        sendRequest({ method: prop, params, id });
                        const promise = new Promise((resolve, reject) => {
                            _self._requestCallbacks.set(id, { resolve, reject });
                            setTimeout(() => reject(new Error(`Request ${prop} timed out`)), timeout);
                        });
                        return promise;
                    };
                }
                else {
                    return params => {
                        throw new Error("Cannot call private function");
                    };
                }
            }
        });
        return proxy;
    }
    _handleSubscribe(...params) {
        const [eventName, callback] = params;
        if (typeof eventName === "string" && typeof callback === "function") {
            let subscriptions = this._subscriptions.get(eventName);
            if (!subscriptions) {
                subscriptions = new Set();
                this._subscriptions.set(eventName, subscriptions);
            }
            subscriptions.add(callback);
        }
    }
    onMessage(message) {
        const eventMessage = message;
        if (eventMessage.eventName) {
            this._onEventMessage(eventMessage);
            return;
        }
        const responseMessage = message;
        if (responseMessage.result || responseMessage.error) {
            this._onRequestResponse(responseMessage);
        }
    }
    _onEventMessage(message) {
        const { id, eventName, params } = message;
        const callbacks = this._subscriptions.get(eventName);
        if (callbacks) {
            callbacks.forEach(callback => callback(...params));
        }
    }
    _onRequestResponse(message) {
        const { id, result, error } = message;
        const callback = this._requestCallbacks.get(id);
        if (callback) {
            this._requestCallbacks.delete(id);
            if (error) {
                callback.reject(error);
            }
            else {
                callback.resolve(result);
            }
        }
    }
}
exports.RemoteProxy = RemoteProxy;
//# sourceMappingURL=RemoteProxy.js.map