"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ws_1 = __importDefault(require("ws"));
const pubsub_room_1 = require("./pubsub-room"); // TODO: исправить
const RemoteProxy_1 = require("../utils/RemoteProxy");
const ServiceWrapper_1 = require("../utils/ServiceWrapper");
const dc_logging_1 = require("@daocasino/dc-logging");
const dc_configs_1 = require("@daocasino/dc-configs");
const logger = new dc_logging_1.Logger("WebSocketTransportProvider");
const defaultSwarm = dc_configs_1.config.default.transportServersSwarm[dc_configs_1.TransportType.WS];
const DEFAULT_PEER_TIMEOUT = dc_configs_1.config.default.waitForPeerTimeout;
const WEB_SOCKET_SERVER = defaultSwarm[0]; // TODO: need select or balance
const randomString = () => Math.random()
    .toString(36)
    .substring(2, 15) +
    Math.random()
        .toString(36)
        .substring(2, 15);
class WebSocketTransportProvider {
    constructor(ws, options) {
        this._options = Object.assign({ waitForPeers: true }, options);
        this._ws = ws;
        this._roomsMap = new Map();
    }
    async _leaveRoom(address) {
        const room = this._roomsMap.get(address);
        if (room) {
            if (await room.leave() === true) {
                return this._roomsMap.delete(address);
            }
        }
        return false;
    }
    async _leaveRooms() {
        for (const address of this._roomsMap.keys()) {
            await this._leaveRoom(address); // TODO: возможно нужно вставить проверку на статус и выдавать error
        }
    }
    async waitForPeer(address, peerId, timeout = DEFAULT_PEER_TIMEOUT) {
        return new Promise((resolve, reject) => {
            const room = this._getWebSocketRoom(address);
            room.once("peer joined", id => {
                // console.log({ address, room })
                if (!peerId || peerId === id) {
                    resolve();
                }
            });
            setTimeout(() => {
                reject(new Error("Waiting for peer timed out"));
            }, timeout);
        });
    }
    static async create() {
        const ws = new ws_1.default(WEB_SOCKET_SERVER);
        return Promise.resolve(new WebSocketTransportProvider(ws));
    }
    async destroy() {
        await this._leaveRooms();
        if (this._ws.readyState === ws_1.default.OPEN) {
            this._ws.close();
            this._ws = null;
        }
        else {
            this._ws.on('open', () => {
                this._ws.close();
                this._ws = null;
            });
        }
    }
    _getWebSocketRoom(address, name) {
        let room = this._roomsMap.get(address);
        if (!room) {
            room = new pubsub_room_1.Room(this._ws, address)
                .on("error", error => {
                logger.error(error);
            })
                .on("peer joined", async (id) => {
                const roomName = `${name || ""} ${address}`;
                const myPeerId = await room.getPeerId();
                logger.debug(`Peer joined ${id} to ${myPeerId} in room ${roomName}`);
            });
            logger.debug(`Room started ${address}`);
            this._roomsMap.set(address, room);
        }
        return room;
    }
    async emitRemote(address, peerId, eventName, params) {
        const room = this._roomsMap.get(address);
        if (!room) {
            throw new Error(`No open room at ${address}`);
        }
        const from = await room.getPeerId();
        const eventMessage = {
            id: RemoteProxy_1.getId(),
            eventName,
            params: [params],
            from
        };
        try {
            await room.sendTo(peerId, JSON.stringify(eventMessage));
        }
        catch (error) {
            throw error;
        }
    }
    getRemoteInterface(address, roomName) {
        const webSocketRoom = this._getWebSocketRoom(address, `Remote interface ${roomName || ""}`);
        const proxy = new RemoteProxy_1.RemoteProxy();
        const self = this;
        webSocketRoom.on("message", async (message) => {
            const peerId = await webSocketRoom.getPeerId();
            if (message.from !== peerId) {
                proxy.onMessage(JSON.parse(message.data));
            }
        });
        return this.waitForPeer(address).then(() => {
            const proxyInterface = proxy.getProxy(message => {
                webSocketRoom.broadcast(JSON.stringify(message));
            });
            // console.log(proxyInterface)
            // const res: any = { v: proxyInterface }
            return Promise.resolve(proxyInterface);
        });
    }
    exposeSevice(address, service, isEventEmitter = false) {
        const webSocketRoom = this._getWebSocketRoom(address, `Expose service ${(service.constructor && service.constructor.name) ||
            ""}`);
        const self = this;
        const wrapper = new ServiceWrapper_1.ServiceWrapper(service, async (response) => {
            const { from, eventName } = response;
            if (eventName) {
                await webSocketRoom.broadcast(JSON.stringify(response));
            }
            else {
                const peerId = await webSocketRoom.getPeerId();
                if (from !== peerId) {
                    try {
                        await webSocketRoom.sendTo(from, JSON.stringify(response));
                    }
                    catch (error) {
                        throw error;
                    }
                }
            }
        }, isEventEmitter);
        if (wrapper.serviceIsEventEmitter) {
            webSocketRoom.on("peer joined", id => {
                service.emit("connected", { id, address });
            });
            webSocketRoom.on("peer left", id => {
                service.emit("disconnected", { id, address });
            });
        }
        webSocketRoom.on("message", async (message) => {
            const { from } = message;
            const peerId = await webSocketRoom.getPeerId();
            if (from !== peerId) {
                const data = JSON.parse(message.data);
                if (data.method) {
                    wrapper.onRequest(Object.assign({}, data, { from }));
                }
            }
        });
    }
    async stopService(address) {
        return this._leaveRoom(address);
    }
}
exports.WebSocketTransportProvider = WebSocketTransportProvider;
//# sourceMappingURL=WebSocketTransportProvider.js.map